       // RESTSRV1: Simple RPG program that supplies a REST web service.
       //
       // This one takes all data from the URL. e.g. a URL such as:
       //   mysystem.com/restsrv1/partnum1/partnum2/.../partnumN
       //   The data being partnum1, partnum2 and so on.
       //
       // Apache configuration directives shown below were used for the mapping
       // ScriptAlias /restsrv1 /qsys.lib/partner400.lib/restsrv1.pgm
       // <Directory /qsys.lib/partner400.lib>
       //   Order Allow,Deny
       //   Allow From All
       // </Directory>

     h Option(*SrcStmt: *NoDebugIO) DftActGrp(*No)
     h BndDir('PARTNER400/CGIDEV2')

     d RestSrv1        pr                  ExtPgm('RESTSRV1')
     d RestSrv1        pi

      // These are the definitons for the response fields
      //   - XML wrapper could be inserted into the DS which would avoid the
      //     need to use the skeleton merge if speed becomes an issue
     d responseDS      ds
     d   partNumber                  10a
     d   description                 30a
     d   quantity                     5i 0

      // Will contain the portion of the URL that follows the identification
      //   i.e the bit that follows http://xxx.com/restsrv1
      // Length of 256 is arbitrary - but if you cane it don't forget to
      //   change the length accepted by ParseURL
     d pathInfo        s            256a   Varying

      // The function ParseURL fills this array from the extracted section of the URL
     d elementList     s             30a   Varying Dim(10)
      // Count returned by ParseURL of number of entries in elementList
     d elementCount    s              5i 0 Inz

     d i               s              5i 0 Inz

      /Free
       // CGIDEV2 Prototypes
      /copy cgidev2/qrpglesrc,prototypeb

       // QUSEC error structure definition
      /copy cgidev2/qrpglesrc,usec

       // Load the xml skeleton into memory - CGIDEV2 identifies the
       //  sections (records) and substitution variable names
       //  second and third parms change the defaults for section markers
       GetHTMLIFS( '/Partner400/RESTSRV1.xml': '<!-- ': ' -->');

       // Write standard response header to buffer
       //  - Note must retain the blank line in the skeleton after content-type
       //  - *off inhibits the output of newline chars
       //  - The '' in the final parm says to output nothing if no value supplied
       //    for one of the substitution variables
       WrtSection('ResponseHeader': *off: '');

       // Use getEnv to grab the back end of the URL
       pathInfo = getEnv('PATH_INFO': qusec );

       // parse the URL string to extract the individual pieces
       elementCount = ParsePath(pathInfo: elementList);

       // Loop through resulting list of URL pieces and set substitution values
       For i = 1 to elementCount;
         // Set variables for response and write section to buffer
         UpdHTMLVar('partNumber': elementList(i));
         UpdHTMLVar('description': 'Description for ' + elementList(i));
         UpdHTMLVar('quantity': %Char( i * 1.33 ));

         WrtSection('responseBody': *off: '');

       EndFor;

       // Response complete so add trailer to buffer then send it all to browser
       WrtSection('responseTrailer *fini': *off: '');

       // Use LR On for testing - for production LR off might be better
       //   performer but some way of shutting down the program required.


       Return;

      /End-Free

      // ParsePath
      // Very prinitive parser that simply looks for "/" characters as
      //   delimiters and breaks up the URL into its component parts.
      // It returns a count of the number of items found.
      // Current max is 10 - but this is just arbitrary. Error checking
      //   should be added to ensure max is not exceeded

     P ParsePath       B
     D                 PI             5i 0
     D  path                        256a   Varying
     D  elements                     30a   Varying Dim(10)

     D position        s              5i 0
     D count           s              5i 0

      // Search starts at position 2 because we know that the first
      //   character in the string will be the / that follows the /restsrv1
      //   part of the URL
     D start           s              5i 0 Inz(2)

      /Free
       DoU position = 0; // Quit when no more to process
         count += 1;
         position = %Scan('/': path: start);
         If position > 0;
           elements(count) = %Subst(path: start: ( position - start));
           start = position + 1;
         Else; // Last element has no trailing /
           elements(count) = %Subst(path: start); // Grab the rest of the string
         EndIf;
       EndDo;

       Return count;

      /End-Free

     P ParsePath       E 